@@ -5,24 +5,35 @@
 from mozphab.conduit import conduit
 from mozphab.helpers import prompt
 from mozphab.logger import logger
+from mozphab.spinner import wait_message
 
 
 def install_certificate(repo, args):
-    """Asks user to enter the Phabricator's certificate.
+    """Asks user to enter the Phabricator API Token.
 
-    The response is saved in the ~/.arcrc file."""
+    Named for parity with arc's corresponding command.
+
+    The response is saved in the ~/.arcrc file; moz-phab itself doesn't do any
+    verification/parsing of the provided string (verification happens by passing
+    it to the Phabricator server)."""
     logger.info(
         "LOGIN TO PHABRICATOR\nOpen this page in your browser and login "
         "to Phabricator if necessary:\n\n%s/conduit/login/\n",
         conduit.repo.phab_url,
     )
     token = prompt("Paste API Token from that page: ")
+
+    # Call a method that requires authentication to both verify the token and clear
+    # the default one-hour expiration of newly created tokens.
+    with wait_message("Verifying token"):
+        who = conduit.whoami(api_token=token)
     conduit.save_api_token(token)
+    logger.info("Configured moz-phab for %s", who["realName"])
 
 
 def add_parser(parser):
     cert_parser = parser.add_parser(
-        "install-certificate", help="Install Phabricator certificate locally"
+        "install-certificate", help="Configure your Phabricator API Token"
     )
     cert_parser.add_argument(
         "--safe-mode",
