@@ -12,6 +12,7 @@ from mozphab.exceptions import Error
 from mozphab.logger import logger
 from mozphab.helpers import augment_commits_from_body, prompt
 from mozphab.spinner import wait_message
+from mozphab.telemetry import telemetry
 
 
 def to_llist(revisions):
@@ -139,6 +140,8 @@ def stack_transactions(remote_phids, local_phids):
 
 
 def reorganise(repo, args):
+    telemetry.metrics.mozphab.submission.preparation_time.start()
+
     with wait_message("Checking connection to Phabricator."):
         # Check if raw Conduit API can be used
         if not conduit.check():
@@ -169,7 +172,8 @@ def reorganise(repo, args):
 
     logger.warning(
         "Reorganisation based on {} commit{}:".format(
-            len(commits), "" if len(commits) == 1 else "s",
+            len(commits),
+            "" if len(commits) == 1 else "s",
         )
     )
 
@@ -220,16 +224,20 @@ def reorganise(repo, args):
                 if t["type"] == "children.set":
                     logger.info(
                         " * {child} will depend on {parent}".format(
-                            child=conduit.phid_to_id(t["value"][0]), parent=node_id,
+                            child=conduit.phid_to_id(t["value"][0]),
+                            parent=node_id,
                         )
                     )
                 if t["type"] == "children.remove":
                     logger.info(
                         " * {child} will no longer depend on {parent}".format(
-                            child=conduit.phid_to_id(t["value"][0]), parent=node_id,
+                            child=conduit.phid_to_id(t["value"][0]),
+                            parent=node_id,
                         )
                     )
 
+    telemetry.metrics.mozphab.submission.preparation_time.stop()
+
     if args.yes:
         pass
     else:
@@ -237,10 +245,13 @@ def reorganise(repo, args):
         if res == "No":
             sys.exit(1)
 
+    telemetry.metrics.mozphab.submission.process_time.start()
+
     with wait_message("Applying transactions..."):
         for phid, rev_transactions in transactions.items():
             conduit.edit_revision(rev_id=phid, transactions=rev_transactions)
 
+    telemetry.metrics.mozphab.submission.process_time.stop()
     logger.info("Stack has been reorganised.")
 
 
@@ -256,14 +267,14 @@ def add_parser(parser):
         "--safe-mode",
         dest="safe_mode",
         action="store_true",
-        help="Run VCS with only necessary extensions.",
+        help="Run VCS with only necessary extensions",
     )
     reorg_parser.add_argument(
         "--upstream",
         "--remote",
         "-u",
         action="append",
-        help='Set upstream branch to detect the starting commit. (default: "")',
+        help='Set upstream branch to detect the starting commit (default: "")',
     )
     reorg_parser.add_argument(
         "start_rev",
