@@ -9,6 +9,12 @@ import re
 import stat
 import sys
 import tempfile
+from typing import (
+    List,
+    Optional,
+    Tuple,
+)
+from itertools import zip_longest
 
 from contextlib import contextmanager
 from shutil import which
@@ -37,7 +43,8 @@ BUG_ID_RE = re.compile(r"(?:(?:bug|b=)(?:\s*)(\d+)(?=\b))", flags=re.IGNORECASE)
 LIST = r"[;,\/\\]\s*"
 LIST_RE = re.compile(LIST)
 IRC_NICK_CHARS = r"a-zA-Z0-9\-\_!"  # includes !, which is different from commitparser
-IRC_NICK = r"#?[" + IRC_NICK_CHARS + "]+"
+IRC_NICK_CHARS_WITH_PERIOD = IRC_NICK_CHARS + r"."
+IRC_NICK = r"#?[" + IRC_NICK_CHARS_WITH_PERIOD + r"]*[" + IRC_NICK_CHARS + r"]+"
 REVIEWERS_RE = (
     r"([\s(.\[;,])(r%s)("
     + IRC_NICK
@@ -51,10 +58,12 @@ ALL_REVIEWERS_RE = re.compile(REVIEWERS_RE % r"[=?]")
 REQUEST_REVIEWERS_RE = re.compile(REVIEWERS_RE % r"[?]")
 GRANTED_REVIEWERS_RE = re.compile(REVIEWERS_RE % r"=")
 R_SPECIFIER_RE = re.compile(r"\br[=?]")
-BLOCKING_REVIEWERS_RE = re.compile(r"\b(r!)([" + IRC_NICK_CHARS + ",]+)")
+BLOCKING_REVIEWERS_RE = re.compile(r"\b(r!)([" + IRC_NICK_CHARS_WITH_PERIOD + ",]+)")
 
 DEPENDS_ON_RE = re.compile(r"^\s*Depends on\s*D(\d+)\s*$", flags=re.MULTILINE)
 
+WIP_RE = re.compile(r"^(?:WIP[: ]|WIP$)", flags=re.IGNORECASE)
+
 
 def which_path(path):
     """Check if an executable is provided. Fall back to which if not.
@@ -229,10 +238,14 @@ def has_arc_rejections(body):
     return all(r.search(body) for r in ARC_REJECT_RE_LIST)
 
 
+def wip_in_commit_title(title):
+    return WIP_RE.search(title) is not None
+
+
 def augment_commits_from_body(commits):
     """Extract metadata from commit body as fields.
 
-    Adds: rev-id, bug-id, reviewers
+    Adds: rev-id, bug-id, reviewers, wip
     """
     for commit in commits:
         commit["rev-id"] = parse_arc_diff_rev(commit["body"])
@@ -251,6 +264,9 @@ def augment_commits_from_body(commits):
         # reviewers
         commit["reviewers"] = parse_reviewers(commit["title"])
 
+        # mark commit as WIP if commit desc starts with "WIP:"
+        commit["wip"] = wip_in_commit_title(commit["title"])
+
     update_commit_title_previews(commits)
 
 
@@ -265,23 +281,37 @@ def parse_reviewers(title):
         "r?" reviewers under the "request" key
         "r=" reviewers under the "granted" key
     """
-    request_reviewers = []
-    for match in re.finditer(REQUEST_REVIEWERS_RE, title):
-        if not match.group(3):
-            continue
-        request_reviewers.extend(re.split(LIST_RE, match.group(3)))
-    granted_reviewers = []
-    for match in re.finditer(GRANTED_REVIEWERS_RE, title):
-        if not match.group(3):
-            continue
-        granted_reviewers.extend(re.split(LIST_RE, match.group(3)))
-    return dict(request=request_reviewers, granted=granted_reviewers)
+
+    def extend_matches(match_re, matches):
+        """Extends `matches` with any matches found using `match_re`.
+        Args:
+            match_re (str): a regular expression string to search with
+            matches (list of str): a list of strings of reviewers captured
+        Returns:
+            dict (str, list of str): a dictionary of requested and granted reviewers
+        """
+        for match in re.finditer(match_re, title):
+            if match.group(3):
+                matches.extend(re.split(LIST_RE, match.group(3)))
+
+    reviewers = {"request": [], "granted": []}
+    extend_matches(REQUEST_REVIEWERS_RE, reviewers["request"])
+    extend_matches(GRANTED_REVIEWERS_RE, reviewers["granted"])
+    return reviewers
 
 
 def strip_depends_on(body):
     return DEPENDS_ON_RE.sub("", body).rstrip()
 
 
+def revision_title_from_commit(commit):
+    """Returns a string suitable for a Revision title for the given commit."""
+    title = WIP_RE.sub("", commit["title-preview"]).lstrip()
+    if commit["wip"]:
+        title = "WIP: %s" % title
+    return title
+
+
 def update_commit_title_previews(commits):
     """Update title-preview from commit metadata for all commits in stack"""
     for commit in commits:
@@ -382,3 +412,107 @@ def short_node(text):
         except ValueError:
             pass
     return text
+
+
+def create_hunk_lines(
+    body: str, prefix: str, check_eof: bool = True
+) -> Tuple[List[str], Optional[bool]]:
+    """Parse a text body into a list of lines to be used in hunks.
+
+    Args:
+        body: A string containing the raw content of the file.
+        prefix: A character (e.g. "+") indicating whether the lines are to be
+            added or removed or are unchanged.
+        check_eof: Whether to check or not to check the end of the file for the
+            newline character. If set to True, the check is performed and if a
+            newline character is missing, it will be added to the last entry in
+            `lines` along with a message.
+
+    Returns:
+        A tuple containing a list of strings to be used when generating the corpus
+            of the hunks, as well as boolean representing whether the file
+            terminated with a new line or not if applicable, `None` if not.
+
+    Note:
+        When the body is missing a newline at the end, the appended lines will use a
+        POSIX line terminator regardless of what line separators are already used in the
+        body.
+    """
+    allowed_prefixes = ("+", "-", " ")
+    if prefix not in allowed_prefixes:
+        raise ValueError(f"Prefix should be one of {allowed_prefixes}")
+
+    empty_file = body in ("", b"")
+
+    if empty_file:
+        # `body` has absolutely nothing in it, return values accordingly.
+        if check_eof:
+            return ["\\ No newline at end of file\n"], True
+        else:
+            return [], None
+
+    # Split lines on line separators
+    lines = split_lines(body)
+    eof_missing_newline = lines[-1] != ""
+
+    # Re-add line separators for all lines except the last line.
+    last_line = lines.pop()
+    lines = join_lineseps(lines)
+    lines = [f"{prefix}{line}" for line in lines]
+    if eof_missing_newline:
+        if check_eof:
+            # Re-add the last line with the line separator, and another line
+            # indicating that no new line was at the EOF.
+            lines.append(f"{prefix}{last_line}\n")
+            lines.append("\\ No newline at end of file\n")
+        else:
+            # Re-add the last line as is since we weren't supposed to touch it.
+            lines.append(f"{prefix}{last_line}")
+
+    return lines, eof_missing_newline if check_eof else None
+
+
+def split_lines(body):
+    """Split a string on line separators, and keep line endings.
+
+    This method behaves the same as `str.splitlines(True)`, but only splits on POSIX
+    and DOS style line endings.
+
+    Args:
+        body: An arbitrary string.
+
+    Returns:
+        A list of strings consisting of lines and line separators.
+
+    Example:
+        >>> test = "line1\nline2\r\nline3"
+        >>> split_lines(test)
+        >>> ["line1", "\n", "line2", "\r\n", "line3"]
+    """
+    binary = isinstance(body, bytes)
+    pattern = b"(\n|\r\n)"
+    if not binary:
+        pattern = pattern.decode("utf-8")
+
+    return re.split(pattern, body)
+
+
+def join_lineseps(lines):
+    """Given a list of strings, join every two entries together where possible.
+
+    NOTE: If the list length is odd, then the last entry is joined with an empty string.
+
+    Args:
+        lines: A list of strings representing lines to join.
+
+    Returns:
+        A list of strings that contains a joined list of strings.
+
+    Example:
+        >>> test = ["line1", "\n", "line2", "\r\n", "line3"]
+        >>> join_lineseps(test)
+        >>> ["line1\n", "line2\r\n", "line3"]
+    """
+    return [
+        "".join(line) for line in zip_longest(lines[0::2], lines[1::2], fillvalue="")
+    ]
