@@ -5,6 +5,7 @@
 import mimetypes
 import os
 import re
+import sys
 import time
 import uuid
 
@@ -19,6 +20,7 @@ from .config import config
 from .diff import Diff
 from .exceptions import CommandError, Error, NotFoundError
 from .helpers import (
+    create_hunk_lines,
     parse_config,
     short_node,
     temporary_binary_file,
@@ -27,7 +29,7 @@ from .helpers import (
 )
 from .logger import logger
 from .repository import Repository
-from .spinner import wait_message
+from .spinner import wait_message, clear_terminal_line
 from .subprocess_wrapper import debug_log_command
 from .telemetry import telemetry
 
@@ -150,11 +152,12 @@ class Mercurial(Repository):
         return not status["T"]
 
     def hg(self, command, **kwargs):
-        self.hg_out(command, **kwargs)
+        self.hg_out(command, capture=False, **kwargs)
 
     def hg_out(
         self,
         command,
+        capture=True,
         expect_binary=False,
         strip=True,
         keep_ends=False,
@@ -162,14 +165,22 @@ class Mercurial(Repository):
         never_log=False,
     ):
         def error_handler(exit_code, stdout, stderr):
-            if stdout and not never_log:
-                logger.debug(stdout.decode())
+            if not capture:
+                clear_terminal_line()
+                if stderr:
+                    print(stderr.decode(), file=sys.stderr, end="")
+                if stdout:
+                    print(stdout.decode(), end="")
 
-            if stderr and not never_log:
-                logger.debug(stderr.decode())
+            if not never_log:
+                if stderr:
+                    logger.debug(stderr.decode().rstrip())
+                if stdout:
+                    logger.debug(stdout.decode().rstrip())
 
             raise CommandError(
-                "command '%s' failed to complete successfully" % command[0], exit_code
+                "command '%s' failed to complete successfully" % command[0].decode(),
+                exit_code,
             )
 
         for arg, value in self._extra_options.items():
@@ -188,7 +199,13 @@ class Mercurial(Repository):
             out = out.rstrip()
         if out and not never_log:
             logger.debug(out)
-        return out.splitlines(keep_ends) if split else out
+
+        if capture:
+            return out.splitlines(keep_ends) if split else out
+
+        clear_terminal_line()
+        print(out, end="")
+        return None
 
     def hg_log(self, revset, split=True, select="node"):
         return self.hg_out(["log", "-T", "{%s}\n" % select, "-r", revset], split=split)
@@ -214,7 +231,7 @@ class Mercurial(Repository):
 
     def after_submit(self):
         # Restore the previously active commit.
-        self.hg(["update", self.previous_bookmark])
+        self.hg(["update", self.previous_bookmark, "--quiet"])
 
     def cleanup(self):
         # Remove the store of obsolescence markers; if the user doesn't have evolve
@@ -666,9 +683,7 @@ class Mercurial(Repository):
             + ["--dest", dest_commit["node"]]
         )
 
-    def check_commits_for_submit(
-        self, commits, validate_reviewers=True, require_bug=True
-    ):
+    def check_commits_for_submit(self, commits, require_bug=True):
         # 'Greatest Common Ancestor'/'Merge Base' should be included in the revset.
         ancestor = self.hg_log("ancestor(%s)" % self.revset, split=False)
         if ancestor not in [c["node"] for c in commits]:
@@ -715,9 +730,7 @@ class Mercurial(Repository):
                 err.append("You can enable the shelve extension via `hg config --edit`")
             raise Error("\n".join(err))
 
-        super().check_commits_for_submit(
-            commits, validate_reviewers=validate_reviewers, require_bug=require_bug
-        )
+        super().check_commits_for_submit(commits, require_bug=require_bug)
 
     def _get_file_modes(self, commit):
         """Get modes of the modified files."""
@@ -914,13 +927,14 @@ class Mercurial(Repository):
             )
             return
 
-        lines = meta["body"].splitlines(keepends=True)
-        lines = ["+%s" % line for line in lines]
-        new_len = len(lines)
+        # emtpy files don't have any hunks
+        if meta["file_size"] == 0:
+            return
 
-        if lines and not lines[-1].endswith("\n"):
-            lines[-1] = "{}\n".format(lines[-1])
-            lines.append("\\ No newline at end of file\n")
+        lines, eof_missing_newline = create_hunk_lines(meta["body"], "+")
+        new_len = len(lines)
+        if eof_missing_newline:
+            new_len -= 1
 
         change.hunks.append(
             Diff.Hunk(
@@ -946,13 +960,11 @@ class Mercurial(Repository):
             )
             return
 
-        lines = meta["body"].splitlines(keepends=True)
-        lines = ["-%s" % line for line in lines]
-
-        if lines and not lines[-1].endswith("\n"):
-            lines[-1] = "{}\n".format(lines[-1])
-            lines.append("\\ No newline at end of file\n")
+        # emtpy files don't have any hunks
+        if meta["file_size"] == 0:
+            return
 
+        lines = create_hunk_lines(meta["body"], "-")[0]
         old_len = len(lines)
 
         change.hunks.append(
@@ -984,8 +996,7 @@ class Mercurial(Repository):
 
         if a_meta["body"] == b_meta["body"]:
             # File contents unchanged.
-            lines = a_meta["body"].splitlines(keepends=True)
-            lines = [" %s" % line for line in lines]
+            lines = create_hunk_lines(a_meta["body"], " ", check_eof=False)[0]
             change.hunks.append(
                 Diff.Hunk(
                     old_off=1,
