@@ -37,6 +37,12 @@ class Repository(object):
         self.call_sign = self._get_setting("repository.callsign")
         self.bmo_url = self._get_setting("bmo_url")
 
+        if self.bmo_url and not (
+            urllib.parse.urlparse(self.bmo_url).scheme == "https"
+            or environment.HTTP_ALLOWED
+        ):
+            raise Error("Only https connections are allowed.")
+
     def is_worktree_clean(self):
         """Check if the working tree is clean."""
 
@@ -158,9 +164,7 @@ class Repository(object):
     def format_patch(self, diff, body, author, author_date):
         """Format a patch appropriate for importing."""
 
-    def check_commits_for_submit(
-        self, commits, *, validate_reviewers=True, require_bug=True
-    ):
+    def check_commits_for_submit(self, commits, *, require_bug=True):
         """Validate the list of commits (from commit_stack) are ok to submit"""
         errors = []
         warnings = []
@@ -191,18 +195,21 @@ class Repository(object):
                 error_msg += "* %s\n" % name
             errors.append(error_msg)
 
-        if validate_reviewers:
-            # Flatten and deduplicate reviewer list, keeping track of the
-            # associated commit
-            for commit in commits:
-                for group in list(commit["reviewers"].keys()):
-                    for reviewer in commit["reviewers"][group]:
-                        all_reviewers.setdefault(group, set())
-                        all_reviewers[group].add(reviewer)
+        # Flatten and deduplicate reviewer list, keeping track of the
+        # associated commit
+        for commit in commits:
+            # We can ignore reviewers on WIP commits, as they won't be passed to Phab
+            if commit["wip"]:
+                continue
 
-                        reviewer = normalise_reviewer(reviewer)
-                        reviewer_commit_map.setdefault(reviewer, [])
-                        reviewer_commit_map[reviewer].append(commit["node"])
+            for group in list(commit["reviewers"].keys()):
+                for reviewer in commit["reviewers"][group]:
+                    all_reviewers.setdefault(group, set())
+                    all_reviewers[group].add(reviewer)
+
+                    reviewer = normalise_reviewer(reviewer)
+                    reviewer_commit_map.setdefault(reviewer, [])
+                    reviewer_commit_map[reviewer].append(commit["node"])
 
         # Verify all reviewers in a single call
         for invalid_reviewer in conduit.check_for_invalid_reviewers(all_reviewers):
@@ -289,7 +296,14 @@ class Repository(object):
 
     def _api_url(self):
         """Return a base URL for conduit API call"""
-        return urllib.parse.urljoin(self.phab_url, "api/")
+        url = urllib.parse.urljoin(self.phab_url, "api/")
+
+        if not (
+            urllib.parse.urlparse(url).scheme == "https" or environment.HTTP_ALLOWED
+        ):
+            raise Error("Only https connections are allowed.")
+
+        return url
 
     @property
     def phab_repo(self):
